// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.


// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/4316fc1aa18bb04678b156f23b22c9d3f996f9c9


// Returns the field usage stats for each field of an index
package fieldusagestats

import (
	gobytes "bytes"
	"context"
	"errors"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"github.com/elastic/elastic-transport-go/v8/elastictransport"
)

const (
	indexMask = iota + 1
)

// ErrBuildPath is returned in case of missing parameters within the build of the request.
var ErrBuildPath = errors.New("cannot build path, check for missing path parameters")

type FieldUsageStats struct {
	transport elastictransport.Interface

	headers http.Header
	values  url.Values
	path    url.URL

	buf *gobytes.Buffer

	paramSet int

	index string
}

// NewFieldUsageStats type alias for index.
type NewFieldUsageStats func(index string) *FieldUsageStats

// NewFieldUsageStatsFunc returns a new instance of FieldUsageStats with the provided transport.
// Used in the index of the library this allows to retrieve every apis in once place.
func NewFieldUsageStatsFunc(tp elastictransport.Interface) NewFieldUsageStats {
	return func(index string) *FieldUsageStats {
		n := New(tp)

		n.Index(index)

		return n
	}
}

// Returns the field usage stats for each field of an index
//
// https://www.elastic.co/guide/en/elasticsearch/reference/master/field-usage-stats.html
func New(tp elastictransport.Interface) *FieldUsageStats {
	r := &FieldUsageStats{
		transport: tp,
		values:    make(url.Values),
		headers:   make(http.Header),
		buf:       gobytes.NewBuffer(nil),
	}

	return r
}

// HttpRequest returns the http.Request object built from the
// given parameters.
func (r *FieldUsageStats) HttpRequest(ctx context.Context) (*http.Request, error) {
	var path strings.Builder
	var method string
	var req *http.Request

	var err error

	r.path.Scheme = "http"

	switch {
	case r.paramSet == indexMask:
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.index))
		path.WriteString("/")
		path.WriteString("_field_usage_stats")

		method = http.MethodGet
	}

	r.path.Path = path.String()
	r.path.RawQuery = r.values.Encode()

	if r.path.Path == "" {
		return nil, ErrBuildPath
	}

	if ctx != nil {
		req, err = http.NewRequestWithContext(ctx, method, r.path.String(), r.buf)
	} else {
		req, err = http.NewRequest(method, r.path.String(), r.buf)
	}

	req.Header.Set("accept", "application/vnd.elasticsearch+json;compatible-with=8")

	if err != nil {
		return req, fmt.Errorf("could not build http.Request: %w", err)
	}

	return req, nil
}

// Do runs the http.Request through the provided transport.
func (r FieldUsageStats) Do(ctx context.Context) (*http.Response, error) {
	req, err := r.HttpRequest(ctx)
	if err != nil {
		return nil, err
	}

	res, err := r.transport.Perform(req)
	if err != nil {
		return nil, fmt.Errorf("an error happened during the FieldUsageStats query execution: %w", err)
	}

	return res, nil
}

// IsSuccess allows to run a query with a context and retrieve the result as a boolean.
// This only exists for endpoints without a request payload and allows for quick control flow.
func (r FieldUsageStats) IsSuccess(ctx context.Context) (bool, error) {
	res, err := r.Do(ctx)

	if err != nil {
		return false, err
	}
	io.Copy(ioutil.Discard, res.Body)
	err = res.Body.Close()
	if err != nil {
		return false, err
	}

	if res.StatusCode >= 200 && res.StatusCode < 300 {
		return true, nil
	}

	return false, nil
}

// Header set a key, value pair in the FieldUsageStats headers map.
func (r *FieldUsageStats) Header(key, value string) *FieldUsageStats {
	r.headers.Set(key, value)

	return r
}

// Index Comma-separated list or wildcard expression of index names used to limit the
// request.
// API Name: index
func (r *FieldUsageStats) Index(v string) *FieldUsageStats {
	r.paramSet |= indexMask
	r.index = v

	return r
}

// AllowNoIndices If false, the request returns an error if any wildcard expression, index
// alias, or _all value targets
// only missing or closed indices. This behavior applies even if the request
// targets other open indices.
// For example, a request targeting `foo*,bar*` returns an error if an index
// starts with `foo` but no index
// starts with `bar`.
// API name: allow_no_indices
func (r *FieldUsageStats) AllowNoIndices(b bool) *FieldUsageStats {
	r.values.Set("allow_no_indices", strconv.FormatBool(b))

	return r
}

// ExpandWildcards Type of index that wildcard patterns can match. If the request can target
// data streams, this argument
// determines whether wildcard expressions match hidden data streams. Supports
// comma-separated values,
// such as `open,hidden`.
// API name: expand_wildcards
func (r *FieldUsageStats) ExpandWildcards(value string) *FieldUsageStats {
	r.values.Set("expand_wildcards", value)

	return r
}

// IgnoreUnavailable If true, missing or closed indices are not included in the response.
// API name: ignore_unavailable
func (r *FieldUsageStats) IgnoreUnavailable(b bool) *FieldUsageStats {
	r.values.Set("ignore_unavailable", strconv.FormatBool(b))

	return r
}

// Fields Comma-separated list or wildcard expressions of fields to include in the
// statistics.
// API name: fields
func (r *FieldUsageStats) Fields(value string) *FieldUsageStats {
	r.values.Set("fields", value)

	return r
}

// MasterTimeout Period to wait for a connection to the master node. If no response is
// received before the timeout expires,
// the request fails and returns an error.
// API name: master_timeout
func (r *FieldUsageStats) MasterTimeout(value string) *FieldUsageStats {
	r.values.Set("master_timeout", value)

	return r
}

// Timeout Period to wait for a response. If no response is received before the timeout
// expires, the request fails
// and returns an error.
// API name: timeout
func (r *FieldUsageStats) Timeout(value string) *FieldUsageStats {
	r.values.Set("timeout", value)

	return r
}

// WaitForActiveShards The number of shard copies that must be active before proceeding with the
// operation. Set to all or any
// positive integer up to the total number of shards in the index
// (`number_of_replicas+1`).
// API name: wait_for_active_shards
func (r *FieldUsageStats) WaitForActiveShards(value string) *FieldUsageStats {
	r.values.Set("wait_for_active_shards", value)

	return r
}
